% view3
%
% Robert A McDougal
% March 2010
%
% A program to illustrate the relationships between 3d graphs and contour
% graphs.
%
% view3(FUNCTION_HANDLE, XMIN, XMAX, YMIN, YMAX)
%
% All arguments are optional.  If no function is specified, default is
% f(x, y) = sqrt(1 - x^2 - y^2)
classdef view3 < handle
    properties
        xmin = -1;
        xmax = 1;
        ymin = -1;
        ymax = 1;
        x_mesh = 50;
        y_mesh = 50;
    end
    
    properties (GetAccess = public, SetAccess = private)
        x;
        y;
        z;
        zmin;
        zmax;
        h;
        fn_ptr = @(x, y) sqrt(1 - x ^ 2 - y ^ 2);
        side_panel;
        z_slider;
        graph_3d;
        current_level;
        contours;
        z_plane;
        add_button;
    end
    
    methods
        % constructor
        function this = view3(fn, xmin, xmax, ymin, ymax)
            if nargin >= 1
                this.fn_ptr = fn;
            end
            if nargin >= 2
                this.xmin = xmin;
            end
            if nargin >= 3
                this.xmax = xmax;
            end
            if nargin >= 4
                this.ymin = ymin;
            end
            if nargin >= 5
                this.ymax = ymax;
            end
            this.compute_graph;
        end
        
        function dsplay(this)
            this.h = figure('Resize', 'on', 'MenuBar', 'none');
            
            % z scroll bar
            this.z_slider = uicontrol('style', 'slider', 'units', 'normalized', ...
                'Position', [.65 0 .049, 1], 'callback', @this.z_slider_callback);
            
            % add contour button
            this.add_button = uicontrol('style', 'pushbutton', ...
                'string', 'Add Contour', 'units', 'normalized', ...
                'position', [.7 0 .3 .2], 'callback', @this.add_button_callback);
            
            % create the axes
            this.graph_3d = axes('parent', this.h, 'position', [.07 .07 .55 .89]);
            view(this.graph_3d, 3);

            
            % create the side graphs
            this.current_level = axes('parent', this.h, 'position', [.75 .65 .2 .3]);
            this.contours = axes('parent', this.h, 'position', [.75 .25 .2 .3]);
            
            % plot graph
            this.plot_graph;
            
            % enable rotation
            rotate3d(this.graph_3d, 'on');
            axis(this.graph_3d, 'vis3d');         
        end
        
        function compute_graph(this)
            % compute the graph
            [this.x, this.y] = meshgrid(linspace(this.xmin, this.xmax, this.x_mesh), ...
                linspace(this.ymin, this.ymax, this.y_mesh));
            this.z = zeros(size(this.x));
            for i = 1 : length(this.x)
                for j = 1 : length(this.y)
                    value = this.fn_ptr(this.x(i, j), this.y(i, j));
                    if ~isreal(value)
                        value = NaN;
                    end
                    this.z(i,j) = value;
                end
            end
            
            this.dsplay;
        end        
    end
    
    methods (Access = protected)
        function plot_graph(this)
            % preserve old view
            [az, el] = view(this.graph_3d);
            
            % redraw graph
            hold(this.graph_3d, 'off');
            s = surfl(this.graph_3d, this.x, this.y, this.z, 'light');
            set(s(1), 'linestyle', 'none');
            set(s(1), 'FaceColor', [.6 0 0]);
            set(s(1), 'facelighting', 'phong');
            xlabel(this.graph_3d, 'x');
            ylabel(this.graph_3d, 'y');
            zlabel(this.graph_3d, 'z');
            
            % create dummy plane
            hold(this.graph_3d, 'on');
            this.z_plane = surfl(this.graph_3d, this.x, this.y, ones(size(this.x)) * mean(zlim(this.graph_3d)));
            set(this.z_plane, 'linestyle', 'none');
            set(this.z_plane, 'FaceColor', [0 0 .6]);
            hold(this.graph_3d, 'off');
            
            this.update_contour;
            
            % reset view
            view(this.graph_3d, az, el);
            
            % set contour graph limits
            xlim(this.contours, [this.xmin this.xmax]);
            ylim(this.contours, [this.ymin this.ymax]);
        end
        
        function update_contour(this)
            % move the plane
            z_value = this.z_level;
            set(this.z_plane, 'zdata', ones(size(this.x)) * z_value);

            % current level contour
            contour(this.current_level, this.x, this.y, this.z, [z_value z_value], 'color', 'black');
        end
        
        function result = z_level(this)
            zl = zlim(this.graph_3d);
            this.zmin = min(zl);
            this.zmax = max(zl);
            result = this.zmin + (this.zmax - this.zmin) * get(this.z_slider, 'value');
        end
        
        function add_button_callback(this, h, e) %#ok<INUSD> call sends h and e
            z_value = this.z_level;
            hold(this.contours, 'on');
            contour(this.contours, this.x, this.y, this.z, [z_value z_value]);
            hold(this.contours, 'off');
        end
        
        % replot the graph with the new slider value, preserve zlim
        function z_slider_callback(this, h, e) %#ok<INUSD> call sends h and e
            zl = zlim(this.graph_3d);
            this.update_contour;
            zlim(this.graph_3d, zl);
        end
    end
end